from urllib.parse import urlparse
from datetime import datetime
from collections import defaultdict

import pytz
from dateutil.parser import parse

from asgard.sdk.options import get_option
import requests

def get_fluentd_server_addresses():
    return get_option("FLUENTD", "ADDRESS")

def get_fluentd_plugin_info(plugin_id):
    result = {}
    now  =  datetime.now(tz=pytz.utc)
    for addr in get_fluentd_server_addresses():
        server_ip = urlparse(addr).hostname
        result[server_ip] = {}
        response = requests.get(f"{addr}/api/plugins.json", timeout=2)
        response_data = response.json()
        plugin_data = [info for info in response_data['plugins'] if info['plugin_id'] == plugin_id]
        if plugin_data:
            if plugin_data[0]['retry']:
                plugin_data[0]['retry_start_min'] = (parse(plugin_data[0]['retry']['start']) - now).total_seconds() / 60
                plugin_data[0]['retry_next_min'] = (parse(plugin_data[0]['retry']['next_time']) - now).total_seconds() / 60
            else:
                plugin_data[0]['retry_start_min'] = 0
                plugin_data[0]['retry_next_min'] = 0

            result[server_ip] = plugin_data[0]
    return result

def get_fluentd_summary_plugin_info(plugin_id):
    """
      "retry_count": each(<IP>).sum(retry_count),
      "buffer_queue_length": each(<IP>).sum(buffer_queue_length),
      "buffer_total_queued_size": each(<IP>).sum(buffer_total_queued_size),
    """
    result = defaultdict(int)
    plugin_info = get_fluentd_plugin_info(plugin_id)
    for server_ip, plugin_data in plugin_info.items():
        result['retry_count'] += plugin_data['retry_count']
        result['buffer_queue_length'] += plugin_data['buffer_queue_length']
        result['buffer_total_queued_size'] += plugin_data['buffer_total_queued_size']
    return result



import json
from datetime import datetime
from dateutil.parser import parse
from dateutil.relativedelta import relativedelta


from flask import Response
import pytz

from fluentdmetrics.blueprint import fluentd_metrics_blueprint
from fluentdmetrics import fluentd

@fluentd_metrics_blueprint.route('/plugins/<string:plugin_id>')
def one_plugin(plugin_id):
    plugin_info = fluentd.get_fluentd_plugin_info(plugin_id)
    plugin_exist = any([plugin_data for plugin_id, plugin_data in plugin_info.items()])
    if plugin_exist:
        final_result = {}
        for server_ip, plugin_data in plugin_info.items():
            now = datetime.now(pytz.utc)
            final_result.update({
                 f"retry_start_min_{server_ip}": plugin_data['retry_start_min'],
                 f"retry_next_min_{server_ip}": plugin_data['retry_next_min'],
                 f"buffer_queue_length_{server_ip}" : plugin_data['buffer_queue_length'],
                 f"retry_count_{server_ip}" : plugin_data['retry_count'],
                 f"buffer_total_queued_size_{server_ip}" : plugin_data['buffer_total_queued_size'],
            })
        return Response(
            json.dumps(final_result),
            mimetype='application/json'
        )
    return Response(status=404, mimetype='application/json')

@fluentd_metrics_blueprint.route('/retry_count/<string:plugin_id>')
def retry_count(plugin_id):
    return Response(
        json.dumps(fluentd.get_fluentd_summary_plugin_info(plugin_id)),
        mimetype='application/json'
    )



from setuptools import setup, find_packages  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name="asgard-api-plugin-metrics-fluentd",
    version="0.2.0",
    description="Asgard API endpoints to get Fluentd metrics",
    long_description="Plugin para a Asgard API e que fornece métricas do cluster de Fluentd",
    url="https://github.com/B2W-BIT/asgard-api-plugin-metrics-fluentd",
    # Author details
    author="Dalton Barreto",
    author_email="daltonmatos@gmail.com",
    license="MIT",
    classifiers=["Programming Language :: Python :: 3.6"],
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    entry_points={
        "asgard_api_metrics_mountpoint": [
            "init = fluentdmetrics.plugin:asgard_api_plugin_init"
        ]
    },
)
